﻿using System;
using System.Collections.Generic;
using System.Linq;
using Nop.Core;
using Nop.Core.Caching;
using Nop.Core.Data;
using Nop.Plugin.Shipping.ByTotal.Domain;

namespace Nop.Plugin.Shipping.ByTotal.Services
{
    /// <summary>
    /// Shipping By Total Service
    /// </summary>
    public partial class ShippingByTotalService : IShippingByTotalService
    {
        #region Constants

        private const string SHIPPINGBYTOTAL_ALL_KEY = "Nop.shippingbytotal.all";
        private const string SHIPPINGBYTOTAL_PATTERN_KEY = "Nop.shippingbytotal.";

        #endregion

        #region Fields

        private readonly IRepository<ShippingByTotalRecord> _sbtRepository;
        private readonly ICacheManager _cacheManager;

        #endregion

        #region Ctor

        /// <summary>
        /// Ctor
        /// </summary>
        /// <param name="cacheManager">Cache manager</param>
        /// <param name="sbtRepository">ShippingByTotal Repository</param>
        public ShippingByTotalService(ICacheManager cacheManager,
            IRepository<ShippingByTotalRecord> sbtRepository)
        {
            this._cacheManager = cacheManager;
            this._sbtRepository = sbtRepository;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Gets all the ShippingByTotalRecords
        /// </summary>
        /// <param name="pageIndex">page index</param>
        /// <param name="pageSize">page size</param>
        /// <returns>ShippingByTotalRecord collection</returns>
        public virtual IPagedList<ShippingByTotalRecord> GetAllShippingByTotalRecords(int pageIndex = 0, int pageSize = int.MaxValue)
        {
            string key = SHIPPINGBYTOTAL_ALL_KEY;
            return _cacheManager.Get(key, () =>
            {
                var query = from sbt in _sbtRepository.Table
                            orderby sbt.StoreId, sbt.CountryId, sbt.StateProvinceId, sbt.Zip, sbt.ShippingMethodId, sbt.From
                            select sbt;

                var records = new PagedList<ShippingByTotalRecord>(query, pageIndex, pageSize);

                return records;
            });
        }

        /// <summary>
        /// Finds the ShippingByTotalRecord by its identifier
        /// </summary>
        /// <param name="shippingByTotalRecordId">ShippingByTotalRecord identifier</param>
        /// <returns>ShippingByTotalRecord</returns>
        public virtual ShippingByTotalRecord GetShippingByTotalRecordById(int shippingByTotalRecordId)
        {
            if (shippingByTotalRecordId == 0)
            {
                return null;
            }

            var record = _sbtRepository.GetById(shippingByTotalRecordId);

            return record;
        }

        /// <summary>
        /// Finds the ShippingByTotalRecord
        /// </summary>
        /// <param name="shippingMethodId">shipping method identifier</param>
        /// <param name="storeId">store identifier</param>
        /// <param name="subtotal">order's subtotal</param>
        /// <param name="countryId">country identifier</param>
        /// <param name="subtotal">subtotal</param>
        /// <param name="stateProvinceId">state province identifier</param>
        /// <param name="zip">Zip code</param>
        /// <returns>ShippingByTotalRecord</returns>
        public virtual ShippingByTotalRecord FindShippingByTotalRecord(int shippingMethodId, int storeId,
            int countryId, decimal subtotal, int stateProvinceId, string zip)
        {
            if (zip == null)
            {
                zip = string.Empty;
            }
            else
            {
                zip = zip.Trim();
            }

            //filter by shipping method and subtotal
            var existingRates = GetAllShippingByTotalRecords()
                .Where(sbt => sbt.ShippingMethodId == shippingMethodId && subtotal >= sbt.From && subtotal <= sbt.To)
                .ToList();

            //filter by store
            var matchedByStore = new List<ShippingByTotalRecord>();
            foreach (var sbt in existingRates)
            {
                if (storeId == sbt.StoreId)
                {
                    matchedByStore.Add(sbt);
                }
            }
            if (matchedByStore.Count == 0)
            {
                foreach (var sbt in existingRates)
                {
                    if (sbt.StoreId == 0)
                    {
                        matchedByStore.Add(sbt);
                    }
                }
            }

            //filter by country
            var matchedByCountry = new List<ShippingByTotalRecord>();
            foreach (var sbt in matchedByStore)
            {
                if (countryId == sbt.CountryId)
                {
                    matchedByCountry.Add(sbt);
                }
            }
            if (matchedByCountry.Count == 0)
            {
                foreach (var sbt in matchedByStore)
                {
                    if (sbt.CountryId == 0)
                    {
                        matchedByCountry.Add(sbt);
                    }
                }
            }

            //filter by state/province
            var matchedByStateProvince = new List<ShippingByTotalRecord>();
            foreach (var sbt in matchedByCountry)
            {
                if (stateProvinceId == sbt.StateProvinceId)
                {
                    matchedByStateProvince.Add(sbt);
                }
            }
            if (matchedByStateProvince.Count == 0)
            {
                foreach (var sbt in matchedByCountry)
                {
                    if (sbt.StateProvinceId == 0)
                    {
                        matchedByStateProvince.Add(sbt);
                    }
                }
            }

            //filter by zip
            var matchedByZip = new List<ShippingByTotalRecord>();
            foreach (var sbt in matchedByStateProvince)
            {
                if ((String.IsNullOrEmpty(zip) && String.IsNullOrEmpty(sbt.Zip)) ||
                    (zip.Equals(sbt.Zip, StringComparison.InvariantCultureIgnoreCase)))
                {
                    matchedByZip.Add(sbt);
                }
            }

            if (matchedByZip.Count == 0)
            {
                foreach (var taxRate in matchedByStateProvince)
                {
                    if (String.IsNullOrWhiteSpace(taxRate.Zip))
                    {
                        matchedByZip.Add(taxRate);
                    }
                }
            }

            return matchedByZip.FirstOrDefault();
        }

        /// <summary>
        /// Deletes the ShippingByTotalRecord
        /// </summary>
        /// <param name="shippingByTotalRecord">ShippingByTotalRecord</param>
        public virtual void DeleteShippingByTotalRecord(ShippingByTotalRecord shippingByTotalRecord)
        {
            if (shippingByTotalRecord == null)
            {
                throw new ArgumentNullException("shippingByTotalRecord");
            }

            _sbtRepository.Delete(shippingByTotalRecord);

            _cacheManager.RemoveByPattern(SHIPPINGBYTOTAL_PATTERN_KEY);
        }

        /// <summary>
        /// Inserts the ShippingByTotalRecord
        /// </summary>
        /// <param name="shippingByTotalRecord">ShippingByTotalRecord</param>
        public virtual void InsertShippingByTotalRecord(ShippingByTotalRecord shippingByTotalRecord)
        {
            if (shippingByTotalRecord == null)
            {
                throw new ArgumentNullException("shippingByTotalRecord");
            }

            _sbtRepository.Insert(shippingByTotalRecord);

            _cacheManager.RemoveByPattern(SHIPPINGBYTOTAL_PATTERN_KEY);
        }

        /// <summary>
        /// Updates the ShippingByTotalRecord
        /// </summary>
        /// <param name="shippingByTotalRecord">ShippingByTotalRecord</param>
        public virtual void UpdateShippingByTotalRecord(ShippingByTotalRecord shippingByTotalRecord)
        {
            if (shippingByTotalRecord == null)
            {
                throw new ArgumentNullException("shippingByTotalRecord");
            }

            _sbtRepository.Update(shippingByTotalRecord);

            _cacheManager.RemoveByPattern(SHIPPINGBYTOTAL_PATTERN_KEY);
        }

        #endregion
    }
}
